@@ -8,7 +8,7 @@ use Data::Alias ();
 use Scope::Guard;
 use Sub::Name;
 
-our $VERSION = '0.10';
+our $VERSION = '0.11';
 
 
 =head1 NAME
@@ -69,19 +69,27 @@ signature.
 Future releases will add extensively to the signature syntax probably
 along the lines of Perl 6.
 
+=head3 C<@_>
+
+Other than removing C<$self>, C<@_> is left intact.  You are free to
+use C<@_> alongside the arguments provided by Method::Signatures.
+
 
 =head3 Aliased references
 
-A signature of C<\@foo> will take an array reference but allow it to
-be used as C<@foo> inside the method.
+A signature of C<\@arg> will take an array reference but allow it to
+be used as C<@arg> inside the method.  C<@arg> is an alias to the
+original reference.  Any changes to C<@arg> will effect the original
+reference.
 
     package Stuff;
-    method foo(\@foo, \@bar) {
-        print "Foo:  @foo\n";
-        print "Bar:  @bar\n";
+    method add_one(\@foo) {
+        $_++ for @foo;
     }
 
-    Stuff->foo([1,2,3], [4,5,6]);
+    my @bar = (1,2,3);
+    Stuff->add_one(\@bar);  # @bar is now (2,3,4)
+
 
 =head3 Invocant parameter
 
@@ -96,6 +104,9 @@ parameter.  Put a colon after it instead of a comma.
         $class->things($arg, $another);
     }
 
+Signatures have an implied default of C<$self:>.
+
+
 =head3 Defaults
 
 Each parameter can be given a default with the C<$arg = EXPR> syntax.
@@ -112,6 +123,12 @@ Passing in C<undef> will override the default.  That means...
     Class->add(99);          # $this = 99, $that = 42
     Class->add(99, undef);   # $this = 99, $that = undef
 
+Earlier parameters may be used in later defaults.
+
+    method copy_cat($this, $that = $this) {
+        return $that;
+    }
+
 All variables with defaults are considered optional.
 
 
@@ -128,6 +145,18 @@ Any unknown trait is ignored.
 Currently there are no traits.  It's for forward compatibility.
 
 
+=head3 Traits and defaults
+
+To have a parameter which has both a trait and a default, set the
+trait first and the default second.
+
+    method echo($message is ro = "what?") {
+        return $message
+    }
+
+Think of it as C<$message is ro> being the left-hand side of the assignment.
+
+
 =head3 Optional parameters
 
 To declare a parameter optional, use the C<$arg?> syntax.
@@ -135,12 +164,31 @@ To declare a parameter optional, use the C<$arg?> syntax.
 Currently nothing is done with this.  It's for forward compatibility.
 
 
+=head3 Required parameters
+
+To declare a parameter as required, use the C<$arg!> syntax.
+
+All parameters without defaults are required by default.
+
+
 =head3 The C<@_> signature
 
 The @_ signature is a special case which only shifts C<$self>.  It
 leaves the rest of C<@_> alone.  This way you can get $self but do the
 rest of the argument handling manually.
 
+
+=head2 Anonymous Methods
+
+An anonymous method can be declared just like an anonymous sub.
+
+    my $method = method ($arg) {
+        return $self->foo($arg);
+    };
+
+    $obj->$method(42);
+
+
 =cut
 
 sub import {
@@ -230,6 +278,7 @@ sub import {
 
             my($sigil, $name) = $proto =~ m{^ (.)(.*) }x;
             $sig->{is_optional} = ($name =~ s{\?$}{} or $sig->{default});
+            $sig->{is_required} = ($name =~ s{\!$}{} or !$sig->{is_optional});
             $sig->{sigil}       = $sigil;
             $sig->{name}        = $name;
         }
@@ -261,7 +310,7 @@ sub import {
             my $rhs = (!$sig->{is_ref_alias} and $sig->{sigil} =~ /^[@%]$/) ? "\@_[$idx..\$#_]" : "\$_[$idx]";
 
             # Handle a default value
-            $rhs = "\@_ > $idx ? $rhs : $sig->{default}" if defined $sig->{default};
+            $rhs = "(\@_ > $idx) ? ($rhs) : ($sig->{default})" if defined $sig->{default};
 
             # XXX We don't do anything with traits right now
 
@@ -284,10 +333,17 @@ sub import {
         my $inject = shift;
         skipspace;
         my $linestr = Devel::Declare::get_linestr;
-        if (substr($linestr, $Offset, 1) eq '{') {
-            substr($linestr, $Offset+1, 0) = $inject;
-            Devel::Declare::set_linestr($linestr);
+
+        if (substr($linestr, $Offset, 1) eq ':') {      # sub with attributes
+            substr($linestr, $Offset, 0) = "sub ";
+            my $block_pos = rindex($linestr, "{");
+            substr($linestr, $block_pos + 1, 0) = $inject;
         }
+        elsif (substr($linestr, $Offset, 1) eq '{') {      # without attributes
+            substr($linestr, $Offset+1, 0) = $inject;
+        }
+
+        Devel::Declare::set_linestr($linestr);
     }
 
     sub scope_injector_call {
@@ -354,7 +410,8 @@ web interface at L<http://rt.cpan.org>.  Report early, report often.
 =head2 C<method> is not declared at compile time.
 
 Unlike declaring a C<sub>, C<method> currently does not happen at
-compile time.  This usually isn't a problem.  We're looking to fix it.
+compile time.  This usually isn't a problem.  It may change, but it
+may be a good thing.
 
 =head2 Debugging
 
@@ -363,7 +420,7 @@ This totally breaks the debugger.  Will have to wait on Devel::Declare fixes.
 =head2 One liners
 
 If you want to write "use Method::Signatures" in a one-liner, do a
-C<-MMethod::Signatures> first.  This is due to a bug in
+C<-MMethod::Signatures> first.  This is due to a bug/limitation in
 Devel::Declare.
 
 =head2 No source filter
@@ -401,10 +458,6 @@ the signature handler pluggable.
 Currently there is no support for types or declaring the type of the
 return value.
 
-=head2 What about anonymous methods?
-
-...what would an anonymous method do?
-
 =head2 How does this relate to Perl's built-in prototypes?
 
 It doesn't.  Perl prototypes are a rather different beastie from
@@ -419,6 +472,11 @@ C<$arg is ro> and C<$arg is alias> respectively, mirroring Perl 6.
 
 Method traits are in the pondering stage.
 
+An API to query a method's signature is in the pondering stage.
+
+Now that we have method signatures, multi-methods are a distinct possibility.
+
+
 =head1 THANKS
 
 This is really just sugar on top of Matt Trout's L<Devel::Declare> work.
